@@ -5,29 +5,34 @@
 <contact>Blaz Zupan (blaz.zupan(@at@)fri.uni-lj.si)</contact>
 <priority>200</priority>
 """
-#
-# OWTestLearners.py
-#
-from OWWidget import *
-import orngTest, orngStat, OWGUI
+
 import time
 import warnings
+import itertools
+
+from OWWidget import *
+
+import orngTest, orngStat, OWGUI
+
 from orngWrap import PreprocessedLearner
-warnings.filterwarnings("ignore", "'id' is not a builtin attribute",
-                        orange.AttributeWarning)
 
 import Orange
 
 ##############################################################################
 
 class Learner:
+    counter = itertools.count()
+
     def __init__(self, learner, id):
         self.learner = learner
         self.name = learner.name
         self.id = id
         self.scores = []
         self.results = None
-        self.time = time.time() # used to order the learners in the table
+        # Used to order the learners in the table (named time for
+        # back compatibility reasons)
+        self.time = next(self.counter)
+
 
 class Score:
     def __init__(self, name, label, f, show=True, cmBased=False):
@@ -36,7 +41,8 @@ class Score:
         self.f = f
         self.show = show
         self.cmBased = cmBased
-        
+
+
 def dispatch(score_desc, res, cm):
     """ Dispatch the call to orngStat method.
     """
@@ -115,6 +121,9 @@ class OWTestLearners(OWWidget):
         self.results = None             # from orngTest
         self.preprocessor = None
 
+        # should the output evaluation results be recomputed
+        self._resultsInvalid = False
+
         self.controlArea.layout().setSpacing(8)
         # GUI
         self.sBtns = OWGUI.radioButtonsInBox(self.controlArea, self, "resampling", 
@@ -314,11 +323,9 @@ class OWTestLearners(OWWidget):
 
     def score(self, ids):
         """compute scores for the list of learners"""
-        if (not self.data):
-            for id in ids:
-                self.learners[id].results = None
-                self.learners[id].scores = []
+        if not self.data:
             return
+
         # test which learners can accept the given data set
         # e.g., regressions can't deal with classification data
         learners = []
@@ -337,6 +344,11 @@ class OWTestLearners(OWWidget):
                 learner = self.preprocessor.wrapLearner(learner)
             try:
                 predictor = learner(new)
+            except Exception, ex:
+                learner_exceptions.append((l, ex))
+                l.scores = []
+                l.results = None
+            else:
                 if (multilabel and isinstance(learner, Orange.multilabel.MultiLabelLearner)) or predictor(new[0]).varType == new.domain.classVar.varType:
                     learners.append(learner)
                     used_ids.append(l.id)
@@ -344,16 +356,11 @@ class OWTestLearners(OWWidget):
                     l.scores = []
                     l.results = None
 
-            except Exception, ex:
-                learner_exceptions.append((l, ex))
-                l.scores = []
-                l.results = None
-
         if learner_exceptions:
             text = "\n".join("Learner %s ends with exception: %s" % (l.name, str(ex)) \
                              for l, ex in learner_exceptions)
             self.warning(0, text)
-            
+
         if not learners:
             return
 
@@ -446,6 +453,12 @@ class OWTestLearners(OWWidget):
         self.paintscores()
         
     def clearScores(self, ids=None):
+        """
+        Clear/invalidate evaluation results/scores for learners for an
+        `ids` sequence (keys into self.learners). If `ids` is None then
+        invalidate data for all learners.
+
+        """
         if ids is None:
             ids = self.learners.keys()
 
@@ -453,41 +466,50 @@ class OWTestLearners(OWWidget):
             self.learners[id].scores = []
             self.learners[id].results = None
 
+        self._resultsInvalid = bool(ids)
+
     # handle input signals
     def setData(self, data):
-        """handle input train data set"""
+        """
+        Set the input train data set.
+        """
         self.closeContext()
-        
-        multilabel= self.ismultilabel(data)
+
+        self.clearScores()
+
+        multilabel = self.ismultilabel(data)
         if not multilabel:
-            self.data = self.isDataWithClass(data, checkMissing=True) and data or None
+            if self.isDataWithClass(data, checkMissing=True):
+                self.data = orange.Filter_hasClassValue(data)
+            else:
+                self.data = None
         else:
             self.data = data
-        
+
         self.fillClassCombo()
-        if not self.data:
-            # data was removed, remove the scores
-            self.clearScores()
-            self.send("Evaluation Results", None)
-        else:
-            # new data has arrived
-            self.clearScores()
 
-            if not multilabel:
-                self.data = orange.Filter_hasClassValue(self.data)
+        if self.data is not None:
+            # Ensure the correct statistics selection is visible.
+            if self.ismultilabel():
+                statwidget = self.mbox
+                stat = self.mStatistics
+            elif self.isclassification():
+                statwidget = self.cbox
+                stat = self.cStatistics
+            else:
+                statwidget = self.rbox
+                stat = self.rStatistics
 
-            self.statLayout.setCurrentWidget([self.rbox, self.cbox, self.mbox][2 if self.ismultilabel() else self.isclassification()])
+            self.statLayout.setCurrentWidget(statwidget)
 
-            self.stat = [self.rStatistics, self.cStatistics, self.mStatistics][2 if self.ismultilabel() else self.isclassification()]
+            self.stat = stat
 
-            if self.learners:
-                self.recompute()
-            
-        self.openContext("", data)
-        self.paintscores()
+            self.openContext("", self.data)
 
     def setTestData(self, data):
-        """handle test data set"""
+        """
+        Set the 'Separate Test Data' input.
+        """
         if data is None:
             self.testdata = None
         else:
@@ -495,16 +517,12 @@ class OWTestLearners(OWWidget):
         self.testDataBtn.setEnabled(self.testdata is not None)
         if self.testdata is not None:
             if self.resampling == 4:
-                if self.data:
-                    self.recompute()
-                else:
-                    for l in self.learners.values():
-                        l.scores = []
-                self.paintscores()
+                self.clearScores()
+
         elif self.resampling == 4 and self.data:
             # test data removed, switch to testing on train data
             self.resampling = 3
-            self.recompute()
+            self.clearScores()
 
     def fillClassCombo(self):
         """upon arrival of new data appropriately set the target class combo"""
@@ -522,79 +540,110 @@ class OWTestLearners(OWWidget):
             self.targetClass=0
 
     def setLearner(self, learner, id=None):
-        """add/remove a learner"""
-        if learner: # a new or updated learner
-            if id in self.learners: # updated learner
+        """
+        Set the input learner with `id`.
+        """
+        if learner is not None:
+            # a new or updated learner
+            if id in self.learners:
                 time = self.learners[id].time
                 self.learners[id] = Learner(learner, id)
                 self.learners[id].time = time
-            else: # new learner
-                self.learners[id] = Learner(learner, id)
-            if self.applyOnAnyChange:
-                self.score([id])
+                self.clearScores([id])
             else:
-                self.recompute()
-        else: # remove a learner and corresponding results
+                self.learners[id] = Learner(learner, id)
+                self._resultsInvalid = True
+        else:
+            # remove a learner and corresponding results
             if id in self.learners:
                 res = self.learners[id].results
                 if res and res.numberOfLearners > 1:
+                    # Remove the learner from the shared results instance
                     old_learner = self.learners[id].learner
                     indx = res.learners.index(old_learner)
                     res.remove(indx)
                     del res.learners[indx]
                 del self.learners[id]
-            self.sendResults()
-        self.paintscores()
-        
+
+                self._resultsInvalid = True
+
     def setPreprocessor(self, pp):
         self.preprocessor = pp
-        if self.learners:
-            self.recompute()
+        self.clearScores()
+
+    def handleNewSignals(self):
+        """
+        Update the evaluations/scores after new inputs were received.
+        """
+        def needsupdate(learner_id):
+            return not (self.learners[learner_id].scores or
+                        self.learners[learner_id].results)
+
+        if self.applyOnAnyChange:
+            self.score(filter(needsupdate, self.learners))
+            if self._resultsInvalid:
+                self.sendResults()
+
+            self.applyBtn.setEnabled(False)
+        else:
+            self.applyBtn.setEnabled(True)
+
+        self.paintscores()
+
+        if self.data is None and self._resultsInvalid:
+            self.send("Evaluation Results", None)
+            self._resultsInvalid = False
+
 
     # handle output signals
 
     def sendResults(self):
         """commit evaluation results"""
-        # for each learner, we first find a list where a result is stored
-        # and remember the corresponding index
+        learners = sorted(self.learners.values(),
+                          key=lambda learner: learner.time)
 
-        valid = [(l.results, l.results.learners.index(l.learner))
-                 for l in self.learners.values() if l.scores and l.results]
-            
-        if not (self.data and len(valid)):
+        learners = [learner for learner in learners
+                    if learner.results and learner.scores]
+
+        if self.data is None or len(learners) == 0:
             self.send("Evaluation Results", None)
+            self._resultsInvalid = False
             return
 
-        # find the result set for a largest number of learners
-        # and remove this set from the list of result sets
-        rlist = dict([(l.results,1) for l in self.learners.values() if l.scores and l.results]).keys()
-        rlen = [r.numberOfLearners for r in rlist]
-        results = rlist.pop(rlen.index(max(rlen)))
-        
-        for (i, l) in enumerate(results.learners):
-            if not l in [l.learner for l in self.learners.values()]:
-                results.remove(i)
-                del results.learners[i]
+        # Split combined results by learner/classifier
+        results_by_learner = {}
+        for learner in learners:
+            results = learner.results
+            res_split = orngStat.split_by_classifiers(results)
+            index = results.learners.index(learner.learner)
+            res_single = res_split[index]
+            res_single.learners = [learner.learner]
+            results_by_learner[learner] = res_single
+
+        def add_results(rhs, lhs):
+            assert(lhs.number_of_learners == 1)
+            rhs.add(lhs, 0)
+            rhs.learners.extend(lhs.learners)
+            return rhs
+
+        # Reconstruct the results in the same order as displayed.
+        results = [results_by_learner[learner] for learner in learners]
+        results = reduce(add_results, results)
 
-        for r in rlist:
-            for (i, l) in enumerate(r.learners):
-                learner_id = [l1.id for l1 in self.learners.values() if l1.learner is l][0]
-                if (r, i) in valid:
-                    results.add(r, i)
-                    results.learners.append(r.learners[i])
-                    self.learners[learner_id].results = results
-        self.send("Evaluation Results", results)
         self.results = results
+        self.send("Evaluation Results", results)
+        self._resultsInvalid = False
+        return
 
     # signal processing
 
     def newsampling(self):
         """handle change of evaluation method"""
+        self.clearScores()
         if not self.applyOnAnyChange:
             self.applyBtn.setDisabled(self.applyOnAnyChange)
-        else:
-            if self.learners:
-                self.recompute()
+        elif self.learners:
+            self.recompute()
 
     def newscoreselection(self):
         """handle change in set of scores to be displayed"""
@@ -622,9 +671,20 @@ class OWTestLearners(OWWidget):
             self.recompute(False)
 
     def applyChange(self):
+        """
+        A change of 'Apply on any change' check box.
+        """
+        def needsupdate(learner_id):
+            return not (self.learners[learner_id].scores or
+                        self.learners[learner_id].results)
+
         if self.applyOnAnyChange:
             self.applyBtn.setDisabled(True)
-        
+            pending = filter(needsupdate, self.learners)
+            if pending:
+                self.score(pending)
+                self.paintscores()
+
     def changedTarget(self):
         self.recomputeCM()
 
@@ -635,13 +695,12 @@ if __name__=="__main__":
     a=QApplication(sys.argv)
     ow=OWTestLearners()
     ow.show()
-    a.exec_()
 
-    data1 = orange.ExampleTable(r'../../doc/datasets/voting')
-    data2 = orange.ExampleTable(r'../../golf')
-    datar = orange.ExampleTable(r'../../auto-mpg')
-    data3 = orange.ExampleTable(r'../../sailing-big')
-    data4 = orange.ExampleTable(r'../../sailing-test')
+    data1 = orange.ExampleTable('voting')
+#     data2 = orange.ExampleTable('golf')
+#     datar = orange.ExampleTable('auto-mpg')
+#     data3 = orange.ExampleTable('sailing-big')
+#     data4 = orange.ExampleTable('sailing-test')
     data5 = orange.ExampleTable('emotions')
 
     l1 = orange.MajorityLearner(); l1.name = '1 - Majority'
@@ -662,21 +721,22 @@ if __name__=="__main__":
 
     l5 = Orange.multilabel.BinaryRelevanceLearner()
 
-    testcase = 4
+    testcase = 0
 
     if testcase == 0: # 1(UPD), 3, 4
-        ow.setData(data2)
+        ow.setData(data1)
         ow.setLearner(r5, 5)
         ow.setLearner(l1, 1)
         ow.setLearner(l2, 2)
         ow.setLearner(l3, 3)
+        ow.handleNewSignals()
+
         l1.name = l1.name + " UPD"
         ow.setLearner(l1, 1)
         ow.setLearner(None, 2)
         ow.setLearner(l4, 4)
-#        ow.setData(data1)
-#        ow.setData(datar)
-#        ow.setData(data1)
+        ow.handleNewSignals()
+
     if testcase == 1: # data, but all learners removed
         ow.setLearner(l1, 1)
         ow.setLearner(l2, 2)
@@ -700,4 +760,5 @@ if __name__=="__main__":
         ow.setData(data5)
         ow.setLearner(l5, 6)
 
+    a.exec_()
     ow.saveSettings()
