@@ -1,44 +1,57 @@
- # Widgets cannot be reset to the settings they had at the time of reporting.
- # The reason lies in the OWGUI callback mechanism: callbacks are triggered only
- # when the controls are changed by the user. If the related widget's attribute
- # is changed programmatically, the control is updated but the callback is not
- # called. This is done intentionally and with a very solid reason: it enables us
- # to do multiple changes without, for instance, the widget being redrawn every time.
- # Besides, it would probably lead to cycles or at least a great number of redundant calls. 
- # However, since setting attributes does not trigger callbacks, setting the attributes
- # here would have not other effect than changing the widget's controls and leaving it
- # in undefined (possibly invalid) state. The reason why we do not have these problems
- # in "normal" use of settings is that the context independent settings are loaded only
- # when the widget is initialized and the context dependent settings are retrieved when
- # the new data is sent and the widget "knows" it has to reconfigure.
- # The only solution would be to require all the widgets have a method for updating
- # everything from scratch according to settings. This would require a lot of work, which
- # could even not be feasible. For instance, there are widget which get the data, compute
- # something and discard the data. This is good since it is memory efficient, but it
- # may prohibit the widget from implementing the update-from-the-scratch method.  
- 
- 
-from OWWidget import *
+# Widgets cannot be reset to the settings they had at the time of reporting.
+# The reason lies in the OWGUI callback mechanism: callbacks are triggered only
+# when the controls are changed by the user. If the related widget's attribute
+# is changed programmatically, the control is updated but the callback is not
+# called. This is done intentionally and with a very solid reason: it enables
+# us to do multiple changes without, for instance, the widget being redrawn
+# every time. Besides, it would probably lead to cycles or at least a great
+# number of redundant calls. However, since setting attributes does not trigger
+# callbacks, setting the attributes here would have not other effect than
+# changing the widget's controls and leaving it in undefined (possibly
+# invalid) state. The reason why we do not have these problems in "normal" use
+# of settings is that the context independent settings are loaded only when
+# the widget is initialized and the context dependent settings are retrieved
+# when the new data is sent and the widget "knows" it has to reconfigure.
+# The only solution would be to require all the widgets have a method for
+# updating everything from scratch according to settings. This would require
+# a lot of work, which
+# could even not be feasible. For instance, there are widget which get the
+# data, compute something and discard the data. This is good since it is
+# memory efficient, but it may prohibit the widget from implementing the
+# update-from-the-scratch method.
+
+import os
+import time
+import tempfile
+import shutil
+import re
+import pickle
+import binascii
+import xml.dom.minidom
+
 from OWWidget import *
+
 from PyQt4.QtWebKit import *
 
 from Orange.utils import environ
 
-import os, time, tempfile, shutil, re, shutil, pickle, binascii
-import xml.dom.minidom
-
-report = None
-
 
 def get_instance():
-    """Return the global ReportWindow instance or None if it was not yet
-    initialized.
-
     """
-    if hasattr(qApp, "canvasDlg") and hasattr(qApp.canvasDlg, "reportWindow"):
-        return qApp.canvasDlg.reportWindow
-    else:
-        return report
+    Return the global `ReportWindow` instance.
+    """
+    app = QApplication.instance()
+    if not hasattr(app, "_reportWindow"):
+        report = ReportWindow()
+        app._reportWindow = report
+        app.sendPostedEvents(report, 0)
+
+        app.aboutToQuit.connect(report.removeTemp)
+        # event loop will still process deferred delete events
+        # after aboutToQuit is emitted
+        app.aboutToQuit.connect(report.deleteLater)
+
+    return app._reportWindow
 
 
 def escape(s):
@@ -49,28 +62,28 @@ class MyListWidget(QListWidget):
     def __init__(self, parent, widget):
         QListWidget.__init__(self, parent)
         self.widget = widget
-        
+
     def dropEvent(self, ev):
         QListWidget.dropEvent(self, ev)
         self.widget.rebuildHtml()
 
     def mousePressEvent(self, ev):
         QListWidget.mousePressEvent(self, ev)
-        node = self.currentItem() 
+        node = self.currentItem()
         if ev.button() == Qt.RightButton and node:
             self.widget.nodePopup.popup(ev.globalPos())
 
-    
+
 class ReportWindow(OWWidget):
-    indexfile = os.path.join(environ.widget_install_dir, "report", "index.html")
-    
+    indexfile = os.path.join(environ.widget_install_dir,
+                             "report", "index.html")
+
     def __init__(self):
         OWWidget.__init__(self, None, None, "Report")
         self.dontScroll = False
-        global report
-        report = self
+        self.widgets = []
         self.counter = 0
-        
+
         self.tempdir = tempfile.mkdtemp("", "orange-report-")
 
         self.tree = MyListWidget(self.controlArea, self)
@@ -86,43 +99,41 @@ class ReportWindow(OWWidget):
         self.treeItems = {}
 
         self.reportBrowser = QWebView(self.mainArea)
-        self.reportBrowser.setUrl(QUrl.fromLocalFile(self.indexfile))
-        self.reportBrowser.page().mainFrame().addToJavaScriptWindowObject("myself", self)
+#        self.reportBrowser.setUrl(QUrl.fromLocalFile(self.indexfile))
+        self.reportBrowser.setHtml(open(self.indexfile, "rb").read())
         frame = self.reportBrowser.page().mainFrame()
-        self.javascript = frame.evaluateJavaScript
+        frame.addToJavaScriptWindowObject("myself", self)
         frame.setScrollBarPolicy(Qt.Vertical, Qt.ScrollBarAsNeeded)
+        self.javascript = frame.evaluateJavaScript
+
         self.mainArea.layout().addWidget(self.reportBrowser)
 
         box = OWGUI.widgetBox(self.controlArea)
-#        exportButton = OWGUI.button(box, self, "&Export", self.saveXML)
-#        OWGUI.button(box, self, "&Import", self.loadXML)
         saveButton = OWGUI.button(box, self, "&Save", self.saveReport)
         printButton = OWGUI.button(box, self, "&Print", self.printReport)
         saveButton.setAutoDefault(0)
-        
+
         self.nodePopup = QMenu("Widget")
-        self.showWidgetAction = self.nodePopup.addAction( "Show widget",  self.showActiveNodeWidget)
+        self.showWidgetAction = self.nodePopup.addAction("Show widget", self.showActiveNodeWidget)
         self.nodePopup.addSeparator()
-        #self.renameAction = self.nodePopup.addAction( "&Rename", self.renameActiveNode, Qt.Key_F2)
         self.deleteAction = self.nodePopup.addAction("Remove", self.removeActiveNode, Qt.Key_Delete)
         self.deleteAllAction = self.nodePopup.addAction("Remove All", self.clearReport)
         self.nodePopup.setEnabled(1)
 
         self.resize(900, 850)
-       
-    # this should have been __del__, but it doesn't get called!
+
     def removeTemp(self):
         try:
             shutil.rmtree(self.tempdir)
         except:
             pass
 
-
     def __call__(self, name, data, widgetId, icon, wtime=None):
         if not self.isVisible():
             self.show()
         else:
             self.raise_()
+
         self.counter += 1
         elid = "N%03i" % self.counter
 
@@ -135,24 +146,69 @@ class ReportWindow(OWWidget):
         widnode.name = name
         self.tree.addItem(widnode)
         self.treeItems[elid] = widnode
+
         self.addEntry(widnode)
-        
-       
+
+    def appendReport(self, name, report, sender=None):
+        """
+        Append a report section titled `name` and with html contents
+        `report`. `sender` if specified can be a OWBaseWidget instance
+        that is sending this report.
+
+        """
+        if not self.isVisible():
+            self.show()
+        else:
+            self.raise_()
+
+        if sender is not None:
+            icon = sender.windowIcon()
+            widgetId = sender.widgetId
+        else:
+            icon = QIcon()
+            widgetId = -1
+
+        self.counter += 1
+        elid = "N%03i" % self.counter
+
+        widnode = QListWidgetItem(icon, name, self.tree)
+        widnode.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable | \
+                         Qt.ItemIsDragEnabled | Qt.ItemIsEditable)
+        widnode.elementId = elid
+        widnode.widgetId = widgetId
+        widnode.time = time.strftime("%a %b %d %y, %H:%M:%S")
+        widnode.data = report
+        widnode.name = name
+        self.tree.addItem(widnode)
+
+        self.treeItems[elid] = widnode
+
+        self.addEntry(widnode)
+
     def addEntry(self, widnode, scrollIntoView=True):
-        newEntry = """
-        <div id="%s" onClick="myself.changeItem(this.id);">
-            <a name="%s" />
-            <h1>%s<span class="timestamp">%s</span></h1>
-            <div class="insideh1">
-                %s
-            </div>
-        </div>
-        """ % (widnode.elementId, widnode.elementId, widnode.name, widnode.time, widnode.data)
+        newEntry = (
+            '<div id="%s" onClick="myself.changeItem(this.id);">'
+            '    <a name="%s">'
+            '    <h1>%s<span class="timestamp">%s</span></h1>'
+            '    <div class="insideh1">'
+            '        %s'
+            '    </div>'
+            '</div>'
+        ) % (widnode.elementId, widnode.elementId, widnode.name,
+             widnode.time, widnode.data)
+
         widnode.content = newEntry
-        self.javascript("document.body.innerHTML += '%s'" % escape(newEntry))
+
+        body = self.bodyFrame()
+
+        body.appendInside(newEntry)
+
         if scrollIntoView:
             self.javascript("document.getElementById('%s').scrollIntoView();" % widnode.elementId)
 
+    def bodyFrame(self):
+        main = self.reportBrowser.page().mainFrame()
+        return main.findFirstElement("body")
 
     def selectionChanged(self, current, previous):
         if current:
@@ -163,30 +219,27 @@ class ReportWindow(OWWidget):
                     var newsel = document.getElementById('%s');
                     newsel.className = 'selected';
                     newsel.scrollIntoView();""" % current.elementId)
-#            if not self.dontScroll:
-#                self.javascript("newsel.scrollIntoView(document.getElementById('%s'));" % current.elementId)
             self.showWidgetAction.setEnabled(current.widgetId >= 0)
         if previous:
             self.javascript("document.getElementById('%s').className = '';" % previous.elementId)
-        
-        
+
     def rebuildHtml(self):
-        self.javascript("document.body.innerHTML = ''")
+        self.bodyFrame().setInnerXml("")
         for i in range(self.tree.count()):
             self.addEntry(self.tree.item(i))
+
         selected = self.tree.selectedItems()
         if selected:
             self.selectionChanged(selected[0], None)
-        
-        
-    @pyqtSignature("QString") 
+
+    @pyqtSignature("QString")
     def changeItem(self, elid):
         self.dontScroll = True
         item = self.treeItems[str(elid)]
         self.tree.setCurrentItem(item)
         self.tree.scrollToItem(item)
         self.dontScroll = False
- 
+
     def raiseWidget(self, node):
         for widget in self.widgets:
             if widget.instance.widgetId == node.widgetId:
@@ -194,12 +247,12 @@ class ReportWindow(OWWidget):
         else:
             return
         widget.instance.reshow()
-        
+
     def showActiveNodeWidget(self):
         node = self.tree.currentItem()
         if node:
             self.raiseWidget(node)
-            
+
     re_h1 = re.compile(r'<h1>(?P<name>.*?)<span class="timestamp">')
     def itemChanged(self, node):
         if hasattr(node, "content"):
@@ -216,46 +269,86 @@ class ReportWindow(OWWidget):
     def clearReport(self):
         self.tree.clear()
         self.rebuildHtml()
-        
+
     def printReport(self):
         printer = QPrinter()
         printDialog = QPrintDialog(printer, self)
         printDialog.setWindowTitle("Print report")
+
         if (printDialog.exec_() != QDialog.Accepted):
             return
-        getattr(self.reportBrowser, "print")(printer)
-        
-        
-    def createDirectory(self):
-        tmpPathName = os.tempnam(orange-report)
-        os.mkdir(tmpPathName)
-        return tmpPathName
-    
+
+        self.reportBrowser.print_(printer)
+
     def getUniqueFileName(self, patt):
         for i in xrange(1000000):
             fn = os.path.join(self.tempdir, patt % i)
             if not os.path.exists(fn):
-                return "file:///"+fn, fn
+                return "file:///" + fn, fn
 
-    img_re = re.compile(r'<IMG.*?\ssrc="(?P<imgname>[^"]*)"', re.DOTALL+re.IGNORECASE)
+    img_re = re.compile(r'<IMG.*?\ssrc="(?P<imgname>[^"]*)"',
+                        re.DOTALL + re.IGNORECASE)
     browser_re = re.compile(r'<!--browsercode(.*?)-->')
+
+    def getSaveDir(self):
+        """
+        Return the initial file system path for the 'Save' dialog.
+        """
+        if hasattr(self, "saveDir"):
+            # set by orngCanvas.OrangeCanvasDlg
+            return self.saveDir
+        else:
+            settings = QSettings()
+            savedir = QDesktopServices.storageLocation(
+                QDesktopServices.DocumentsLocation
+            )
+
+            if PYQT_VERSION < 0x40803:
+                savedir = settings.value("OWReport/save-directory",
+                                         defaultValue=savedir).toString()
+            else:
+                savedir = settings.value("OWReport/save-directory",
+                                         defaultValue=savedir,
+                                         type=unicode)
+            return savedir
+
+    def storeSaveDir(self, savedir):
+        """
+        Store the chosen folder path for subsequent save dialog
+        initialization.
+
+        """
+        if hasattr(self, "saveDir"):
+            self.saveDir = savedir
+        else:
+            settings = QSettings()
+            settings.setValue("OWReport/save-directory", savedir)
+
     def saveReport(self):
-        filename = QFileDialog.getSaveFileName(self, "Save Report", self.saveDir, "Web page (*.html *.htm)")
+        """
+        Save the report to a html file chosen by the user.
+        """
+        savedir = self.getSaveDir()
+        filename = QFileDialog.getSaveFileName(self, "Save Report", savedir,
+                                               "Web page (*.html *.htm)")
         filename = unicode(filename)
-        
+
         if not filename:
             return
-        
+
         path, fname = os.path.split(filename)
-        self.saveDir = path
+
+        self.storeSaveDir(path)
+
         if not os.path.exists(path):
             try:
                 os.makedirs(path)
             except:
-                QMessageBox.error(None, "Error", "Cannot create directory "+path)
+                QMessageBox.error(None, "Error",
+                                  "Cannot create directory " + path)
 
         tt = file(self.indexfile, "rt").read()
-        
+
         index = "<br/>".join('<a href="#%s">%s</a>' % (self.tree.item(i).elementId, self.re_h1.search(self.tree.item(i).content).group("name"))
                              for i in range(self.tree.count()))
             
@@ -296,80 +389,8 @@ class ReportWindow(OWWidget):
         if subdir:
             tt = tt.replace(filepref, subdir+"/")
         file(filename, "wb").write(tt.encode("utf8"))
- 
 
-    def saveXML(self):
-        filename = QFileDialog.getSaveFileName(self, "Export Report", self.saveDir, "XML file (*.xml)")
-        filename = unicode(filename)
-        if not filename:
-            return
 
-        outf = file(filename, "wt")
-        outf.write('<?xml version="1.0" encoding="ascii"?>\n<report version="1.0">\n')
-        
-        for i in range(self.tree.count()):
-            item = self.tree.item(i)
-            outf.write('<entry name="%s" time="%s">\n' % (item.name, item.time))
-
-            filepref = "file:///"+self.tempdir
-            if filepref[-1] != os.sep:
-                filepref += os.sep
-            lfilepref = len(filepref)
-            imspos = -1
-            data = item.data
-            while True:
-                imspos = data.find(filepref, imspos+1)
-                if imspos == -1:
-                    break
-                imname = data[imspos+lfilepref:data.find('"', imspos)]
-                fname = os.path.join(filepref[8:], imname)
-                outf.write('    <binary name="%s"><![CDATA[%s]]></binary>\n' % (imname, binascii.b2a_base64(file(fname, "rb").read())))
-               
-            data = data.replace(filepref, "binary:///")
-            outf.write("<content><![CDATA[%s]]></content>\n\n\n" % data)
-            outf.write('</entry>\n')
-        outf.write('</report>')
-        
-
-    def loadXML(self):
-        filename = QFileDialog.getOpenFileName(self, "Import Report", self.saveDir, "XML file (*.xml)")
-        filename = unicode(filename)
-        
-        if not filename:
-            return
-
-        filepref = "file:///"+self.tempdir
-        if 1:#try:
-            x = xml.dom.minidom.parse(file(str(filename)))
-            x.normalize()
-            entries = []
-            files = []
-            for entry in x.getElementsByTagName("entry"):
-                data = entry.getElementsByTagName("content")[0].firstChild.data
-                for fle in entry.getElementsByTagName("binary"):
-                    name = oname = fle.getAttribute("name")
-                    base, ext = os.path.splitext(name)
-                    i = 0
-                    while os.path.exists(os.path.join(self.tempdir, name)):
-                        i += 1
-                        name = "%s%04i%s" % (base, i, ext)
-                    data = data.replace("binary:///"+oname, filepref+"/"+name)
-                    filedata = binascii.a2b_base64(fle.firstChild.data)
-                    files.append((name, filedata)) 
-                name = entry.getAttribute("name")
-                time = entry.getAttribute("time")
-                entries.append((name, data, None, QIcon(), time))
-                
-            for fname, fdata in files:
-                print fname, len(fdata)
-                file(os.path.join(self.tempdir, fname), "wb").write(fdata)
-            for entry in entries:
-                print entry[1]
-                self(*entry)
-        #except:
-            pass
-            # !!!!!!!!!!!
-        
 def getDepth(item, expanded=True):
     ccount = item.childCount()
     return 1 + (ccount and (not expanded or item.isExpanded()) and max(getDepth(item.child(cc), expanded) for cc in range(ccount)))
